import unittest
from pathlib import Path

import pandas as pd
from office.api.excel import *
import os


class TestExcel(unittest.TestCase):
    def test_fake2excel(self):
        fake2excel(language='fdsa')

    def test_split_excel_by_column(self):
        split_excel_by_column(filepath=r'../../contributors/bulabean/sedemo.xls',
                              column=6)

    def test_sheet2excel(self):
        sheet2excel(file_path=r'./test_files/excel/fake2excel.xlsx')

    def test_merge2sheet(self):
        merge2sheet(dir_path=r'./test_files/excel/merge2sheet')

    def test_merge2excel(self):
        merge2excel(dir_path=r'../../contributors/bulabean', output_file='test_merge2excel.xlsx', )

    def test_find_excel_data(self):
        find_excel_data(search_key='刘家站垦殖场', target_dir=r'../../contributors/bulabean')

    # def test_merge2sheet(self):
    #     """
    #     https://blog.csdn.net/xue_11/article/details/118424380
    #     https://www.jb51.net/article/214868.htm
    #     """
    #     dir_path = 'test_files/excel'
    #     for root, dirs, files in os.walk(dir_path):
    #         path = Path(dir_path)
    #         print(files)
    #         df_list = []
    #         for file in files:
    #             if file.endswith("xlsx") or file.endswith("xls"):
    #                 excel_path = (path / file)
    #                 df_list.append(pd.read_excel(excel_path))
    #         res = pd.concat(df_list)
    #         res.to_excel(
    #             R"./excel/output_file2.xlsx",
    #             sheet_name="手机商品",
    #             index=False  # 不保留index
    #         )

    # single_df_1 = pd.read_excel(r'./excel/1月.xls')
    # print(single_df_1)
    # single_df_2 = pd.read_excel(r'./excel/2月.xls')
    # single_df_3 = pd.read_excel(r'./excel/3月.xls')
    # single_df_4 = pd.read_excel(r'./excel/4月.xls')
    # res = pd.concat([single_df_1, single_df_2,single_df_3,single_df_4])
    # res.to_excel(
    #     R"./excel/output_file1.xlsx",
    #     sheet_name="手机商品",
    #     index=False,  # 不保留index
    # )



# pip install python-office -i https://pypi.python.org/simple -U
# pip install PyOfficeRobot -i https://pypi.python.org/simple -U
# pip install poimage -i https://pypi.python.org/simple -U

# 1、pip freeze > allpackages.txt
# 2、pip uninstall -r allpackages.txt -y
# 3、pip install --upgrade python-office


#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: 图片.py
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关 图片 的自动化操作
#############################################
import poimage


# from office.core.ImageType import MainImage
# from office.lib.utils.except_utils import except_dec

# mainImage = MainImage()


# @except_dec()
from office.lib.decorator_utils.instruction_url import instruction

@instruction
def image2gif():
    poimage.image2gif()


# todo：输出文件路径
# @except_dec()
@instruction
def add_watermark(file, mark, output_path=r'./', out="mark_img", color="#8B8B1B", size=30, opacity=0.15, space=75,
                  angle=30):
    poimage.add_watermark(file, mark, output_path, out, color, size, opacity, space, angle)
    # mainImage.add_watermark(file, mark, out, color, size, opacity, space, angle)


# todo：输入文件路径
# @except_dec()
@instruction
def img2Cartoon(path, client_api='OVALewIvPyLmiNITnceIhrYf', client_secret='rpBQH8WuXP4ldRQo5tbDkv3t0VgzwvCN'):
    poimage.img2Cartoon(path, client_api, client_secret)
    # mainImage.img2Cartoon(path, client_api, client_secret)


# @except_dec()
@instruction
def down4img(url, output_path='.', output_name='down4img', type='jpg'):
    poimage.down4img(url, output_path, output_name, type)
    # mainImage.down4img(url, output_name, type)

@instruction
def txt2wordcloud(filename, color="white", result_file="your_wordcloud.png"):
    poimage.txt2wordcloud(filename, color, result_file)

@instruction
def pencil4img(input_img, output_path='./', output_name='pencil4img.jpg'):
    poimage.pencil4img(input_img, output_path, output_name)

@instruction
def decode_qrcode(qrcode_path):
    """
    解析二维码
    :param qrcode_path: 二维码图片的路径
    :return:
    """
    poimage.decode_qrcode(qrcode_path)



import unittest

from office.api.image import *


class TestImage(unittest.TestCase):
    def test_add_watermark(self):
        add_watermark(file='../test_files/images/0816.jpg', mark='python-office',output_path=r'../test_output/img_output')

    def test_down4img(self):
        down4img(url='https://python-office-1300615378.cos.ap-chongqing.myqcloud.com/python-office-qr.jpg',output_path=r'D:\download\xunlei')

    # def test_img2Cartoon(self):
    #     img2Cartoon()

    def test_txt2wordcloud(self):
        txt2wordcloud(filename=r'D:\workplace\code\test\wordcloud\test.txt', color="white",
                      result_file="your_wordcloud.png")

    def test_pencil4img(self):
        pencil4img(input_img=r'D:\workplace\code\test\down4img\girl.jpg')


# from office.core.VideoType import MainVideo
# mainVideo = MainVideo()
import povideo


# 从视频里提取音频
from office.lib.decorator_utils.instruction_url import instruction

@instruction
def video2mp3(path, mp3_name=None, output_path=r'./'):
    povideo.video2mp3(path, mp3_name, output_path)


# 从音频里，提取文字
# 本地语音文件不能大于5MB
@instruction
def audio2txt(audio_path, appid, secret_id, secret_key):
    povideo.audio2txt(audio_path, appid, secret_id, secret_key)



import unittest

from office.api.word import docx2pdf

from office.api.wechat import send_file


class TestWechat(unittest.TestCase):
    def test_docx2pdf(self):
        docx2pdf(path=r'e://info//test//transverse2.docx')



import unittest

from office.api.testApi.ruiming import screen_unmarked_image, change_label_in_xml, screen_without_label_json_file


class TestRuiming(unittest.TestCase):
    def test_screen_unmarked_image(self):
        screen_unmarked_image(dir_path='./test_files/ruiming/screen_unmarked_image')
        # 预期结果：2.jpg被移动到“未标注图片”目录下

    def test_change_label_in_xml(self):
        # TODO：相对路径问题
        change_label_in_xml(dir_path="./test_files/ruiming/change_label_in_xml", old_label="测试", new_label="测试1")
        # 预期结果：name标签内容从“测试”改为“测试1”

    def test_screen_without_label_json_file(self):
        # TODO：相对路径问题
        screen_without_label_json_file(dir_path="./test_files/ruiming/screen_without_label_json_file")
        # 预期结果：除1.json外均被移动到”无标签json文件“文件夹中



import pathlib
import shutil
import xml.etree.ElementTree
import json


class MainRuiming():
    def __make_dir(self, dir_path, dir_name):
        """

        :param dir_path: 准备新建的位置
        :param dir_name: 新建文件夹的名称
        """
        new_dir_path = dir_path.joinpath(dir_name)
        if not new_dir_path.is_dir():
            new_dir_path.mkdir()
        elif list(new_dir_path.iterdir()) != []:
            exit("目录\"" + str(new_dir_path) + "\"存在且不为空，请检查！")

    def screen_unmarked_image(self, dir_path, image_name_extension: str = ".jpg",
                              marked_file_name_extension: str = ".xml"):
        """

        :param dir_path: 图片及标注文件的存放路径
        :param image_name_extension: 图片文件的后缀，默认为.jpg
        :param marked_file_name_extension: 标注文件的后缀，默认为.xml
        """
        dir_path = pathlib.Path(dir_path).resolve()
        if dir_path.is_dir():
            unmarked_image_storage_path = dir_path.joinpath("未标注图片")
            self.__make_dir(dir_path, "未标注图片")
            image_name_root_set = set()
            marked_file_name_root_set = set()
            # 创建集合
            dir_path_file_list = list(dir_path.iterdir())
            for file_name in dir_path_file_list:
                # 按文件类型添加文件名到对应的集合
                if file_name.is_file():
                    if file_name.suffix == image_name_extension:
                        image_name_root_set.add(file_name.name.replace(image_name_extension, ""))
                    if file_name.suffix == marked_file_name_extension:
                        marked_file_name_root_set.add(file_name.name.replace(marked_file_name_extension, ""))
            unmarked_image_list = list(image_name_root_set - marked_file_name_root_set)
            for i in unmarked_image_list:
                shutil.move(dir_path.joinpath(i + image_name_extension),
                            unmarked_image_storage_path.joinpath(i + image_name_extension))
            print("筛选完成")
        else:
            print("路径输入有误，请检查！")

    def change_label_in_xml(self, dir_path, old_label, new_label):
        """

        :param dir_path: 图片及标注文件的存放路径
        :param old_label: 需要修改的标签
        :param new_label: 修改后的标签
        """
        dir_path = pathlib.Path(dir_path).resolve()
        if dir_path.is_dir():
            file_list = list(dir_path.iterdir())
            for file in file_list:
                if file.suffix == ".xml":
                    xml_file = xml.etree.ElementTree.parse(str(file))
                    xml_root = xml_file.getroot()
                    label_xpath = "./object/name"
                    label_list = xml_root.findall(label_xpath)
                    for label in label_list:
                        if label.text == old_label:
                            label.text = new_label
                    xml_file.write(str(file), encoding="utf-8")
        else:
            print("请输入正确的路径！")

    def screen_without_label_json_file(self, dir_path):
        dir_path = pathlib.Path(dir_path).resolve()
        if dir_path.is_dir():
            print("正在筛选无标签内容的json文件")
            without_label_json_storage_path = dir_path.joinpath("无标签json文件")
            self.__make_dir(dir_path, "无标签json文件")
            dir_path_file_list = list(dir_path.iterdir())
            for file_name in dir_path_file_list:
                if file_name.is_file() and file_name.suffix == ".json":
                    json_file = open(file_name, "r")
                    json_file_text = json.load(json_file)
                    json_file.close()
                    if json_file_text["shapes"] == []:
                        shutil.move(dir_path.joinpath(file_name.name),
                                    without_label_json_storage_path.joinpath(file_name.name))
            print("筛选完成")
        else:
            print("路径输入有误，请检查！")



from office.lib.decorator_utils.instruction_url import instruction

from office.lib.utils.except_utils import except_dec

# from office.core.ToolsType import wftools

# wftools = wftools()

import wftools

# @except_dec()
@instruction
def transtools(to_lang, content):
    wftools.transtools(to_lang, content)


# @except_dec()
@instruction
def qrcodetools(url):
    wftools.qrcodetools(url)


# @except_dec()
@instruction
def passwordtools(len=8):
    return wftools.passwordtools(len)


# @except_dec()
@instruction
def weather():
    wftools.weather()


# 通过url，获取ip地址
# # @except_dec()
@instruction
def url2ip(url):
    wftools.url2ip(url)


# 通过url，获取ip地址
# @except_dec()
@instruction
def lottery8ticket():
    wftools.lottery8ticket()


# @except_dec()
@instruction
def create_article(theme, line_num=200):
    wftools.create_article(theme, line_num)


# @except_dec()
@instruction
def pwd4wifi(len_pwd=8, pwd_list=[]):
    wftools.pwd4wifi(len_pwd, pwd_list)

# 测试网速
@instruction
def net_speed_test():
    wftools.net_speed_test()


import unittest

from office.api.wechat import *


class TestWechat(unittest.TestCase):
    def test_send_file(self):
        send_file(who='文件传输助手', file=r'./test_files/images/0816.jpg')

    def test_receive_message(self):
        receive_message(who='程序员晚枫')



# from office.core.MarkdownType import MainMarkdown
# from office.lib.utils.except_utils import except_dec
#
# mainMarkdown = MainMarkdown()
#
#
# # @except_dec()
# def markdown_link_image_to_base64(markdown_path):
#     mainMarkdown.markdown_link_image_to_base64(markdown_path)
#
#
# # @except_dec()
# def check_local_dir_image_link_markdown(markdown_path, image_path):
#     mainMarkdown.check_local_dir_image_link_markdown(markdown_path, image_path)


import PyOfficeRobot
from office.lib.decorator_utils.instruction_url import instruction


@instruction
def send_message(who, message):
    PyOfficeRobot.chat.send_message(who, message)


@instruction
def send_message_by_time(who, message, time):
    PyOfficeRobot.chat.send_message_by_time(who, message, time)


@instruction
def chat_by_keywords(who, keywords):
    PyOfficeRobot.chat.chat_by_keywords(who, keywords)


@instruction
def send_file(who, file):
    PyOfficeRobot.file.send_file(who, file)


# 保存指定人的消息
# BY：盖飞
@instruction
def receive_message(who='文件传输助手', txt='userMessage.txt', output_path='./'):
    PyOfficeRobot.chat.receive_message(who, txt, output_path)



import unittest

import pytest

from office.api.tools import *
from tests.test_utils.test_input import stub_stdin


class TestTools(unittest.TestCase):
    @pytest.mark.skip(reason="交互式功能，暂时跳过")
    def test_weather(self):
        stub_stdin(self, '北京\ny\nq\n')  # 依次输入
        weather()

    def test_url2ip(self):
        url2ip('www.python-office.com')

    @pytest.mark.skip(reason="交互式功能，暂时跳过")
    def test_lottery8ticket(self):
        stub_stdin(self, '12\n0\n')  # 依次输入
        lottery8ticket()

    def test_create_article(self):
        create_article('生日快乐', line_num=2000)

    # def test_pwd4wifi(self):
    #     stub_stdin(self, '1\ny\n')  #依次输入
    #     pwd4wifi(pwd_list=['12345678', 'CoderWanFeng'])



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: excel.py
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关 excel 的自动化操作
#############################################
# from office.core.ExcelType import MainExcel
# mainExcel = MainExcel()

import poexcel


# todo:输出文件路径
# @except_dec()
from office.lib.decorator_utils.instruction_url import instruction


@instruction
def fake2excel(columns=['name'], rows=1, path='./fake2excel.xlsx', language='zh_CN'):
    poexcel.fake2excel(columns, rows, path, language)


# 多个excel，合并到一个excel的不同sheet中
# @except_dec()
@instruction
def merge2excel(dir_path, output_file='merge2excel.xlsx'):
    """
    :param dir_path:
    :param output_file:
    :return:
    """
    poexcel.merge2excel(dir_path, output_file)


# 同一个excel里的不同sheet，拆分为不同的excel文件
# @except_dec()
@instruction
def sheet2excel(file_path, output_path='./'):
    poexcel.sheet2excel(file_path, output_path)


# @except_dec()
@instruction
def merge2sheet(dir_path, output_sheet_name: str = 'Sheet1', output_excel_name: str = 'merge2sheet'):
    poexcel.merge2sheet(dir_path, output_sheet_name, output_excel_name)


# 搜索excel中指定内容的文件、行数、内容详情
# PR内容 & 作者：https://gitee.com/CoderWanFeng/python-office/pulls/10
# @except_dec()
@instruction
def find_excel_data(search_key: str, target_dir: str):
    poexcel.find_excel_data(search_key, target_dir)


# 按指定列的内容，拆分excel
# PR内容 & 作者：：https://gitee.com/CoderWanFeng/python-office/pulls/11
# @except_dec()
@instruction
def split_excel_by_column(filepath: str, column: int, worksheet_name: str = None):
    poexcel.split_excel_by_column(filepath, column, worksheet_name)



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: instruction_url.py
# Mail: 1957875073@qq.com
# Created Time:  2022-12-17 08:14:34
# Description: 有关 方法说明 的装饰器
#############################################

# 每个文件的具体方法说明
from functools import wraps

from office.lib.conf.CONST import SPLIT_LINE

email_dict = {}
import os

excel_dict = {"fake2excel": "https://www.bilibili.com/video/BV1wr4y1b7uk",
              "find_excel_data": "https://www.bilibili.com/video/BV1Bd4y1B7yr",
              "merge2excel": "https://www.bilibili.com/video/BV1714y147Ao",
              "merge2sheet": "",
              "sheet2excel": "https://mp.weixin.qq.com/s/dAx6JEbj5OlVnCcxokCzTQ",
              "split_excel_by_column": "",
              }
image_dict = {"add_watermark": "https://www.bilibili.com/video/BV1jT411T7n9",
              "decode_qrcode": "https://mp.weixin.qq.com/s/Z_RcTRYxUFpCQBGpShO0ig",
              "down4img": "https://mp.weixin.qq.com/s/P6pRm1VX8bGYepC8O4Bt4Q",
              "image2gif": "",
              "img2Cartoon": "https://mp.weixin.qq.com/s/5Eyk2j20jzSaVcr1DTsfvw",
              "pencil4img": "https://www.bilibili.com/video/BV1rP411N7ao",
              "txt2wordcloud": "https://www.bilibili.com/video/BV1Me4y1h7Ku/?spm_id_from=333.999.0.0",
              }
md_dict = {}
ocr_dict = {}
pdf_dict = {"add_img_water": "",
            "add_watermark": "https://www.bilibili.com/video/BV1Se411T7au",
            "add_watermark_by_parameters": "",
            "decrypt4pdf": "https://mp.weixin.qq.com/s/GiXYB_xZdlsYv5AIeIELkA",
            "encrypt4pdf": "https://mp.weixin.qq.com/s/GiXYB_xZdlsYv5AIeIELkA",
            "merge2pdf": "https://mp.weixin.qq.com/s/9erh3W3WCD36Axj70pRvog",
            "pdf2docx": "https://mp.weixin.qq.com/s/eTwtTXLAudRQmyhE4LY_Dg",
            "pdf2imgs": "https://mp.weixin.qq.com/s/tvHZIBGcrMBLTLB4u23Mow",
            "txt2pdf": "", }
ppt_dict = {"ppt2pdf": "https://mp.weixin.qq.com/s/T31F-U5AdDd3D-61b_K5Qg", }
tools_dict = {"create_article": "https://mp.weixin.qq.com/s/guBtZHyUyybGNOqzZke1kQ",
              "lottery8ticket": "https://mp.weixin.qq.com/s/Q6JgW06hMUSik0DqGD0rOA",
              "net_speed_test": "https://www.bilibili.com/video/BV1fD4y1x7ZE",
              "passwordtools": "https://www.bilibili.com/video/BV1Ce4y1p7iV",
              "pwd4wifi": "https://blog.csdn.net/weixin_42321517/article/details/126677254",
              "qrcodetools": "https://www.bilibili.com/video/BV1pd4y1b716/?spm_id_from=333.999.0.0",
              "transtools": "https://www.bilibili.com/video/BV11d4y1k7Mn",
              "url2ip": "https://cloud.tencent.com/developer/article/2022777",
              "weather": "https://mp.weixin.qq.com/s/NVn8NNtOS3AfOyl75JTaNg",
              }
video_dict = {"audio2txt": "https://mp.weixin.qq.com/s/Ug_IFSEQBzDshe7KuFckTQ",
              "video2mp3": "https://mp.weixin.qq.com/s/Ug_IFSEQBzDshe7KuFckTQ",
              }
web_dict = {}
wechat_dict = {"chat_by_keywords": "https://www.bilibili.com/video/BV1fV4y1M7ju",
               "receive_message": "",
               "send_file": "https://www.bilibili.com/video/BV1te4y1y7Ro",
               "send_message": "https://www.bilibili.com/video/BV1Jt4y1j7F1",
               "send_message_by_time": "https://www.bilibili.com/video/BV1m8411b7LZ", }
word_dict = {"docx2pdf": "https://mp.weixin.qq.com/s/eBn3N_FEx1dlC_-ttmlOwg", }

file_dict = {"search_by_content": "https://mp.weixin.qq.com/s/rvU7O3zHJ-YEd2YU0Z4pew",
             "add_line_by_type": "",
             "file_name_add_postfix": "",
             "file_name_add_prefix": "",
             "file_name_insert_content": "",
             "output_file_list_to_excel": "",
             "replace4filename": "https://www.bilibili.com/video/BV12r4y187Yj",
             "search_specify_type_file": "",
             }

# 有多少文件需要说明
instruction_file_dict = {
    "excel.py": excel_dict,
    "image.py": image_dict,
    "pdf.py": pdf_dict,
    "ppt.py": ppt_dict,
    "tools.py": tools_dict,
    "video.py": video_dict,
    "wechat.py": wechat_dict,
    "word.py": word_dict,
    "file.py": file_dict,
}


def instruction(func):
    @wraps(func)
    def instruction_wrapper(*args, **kwargs):
        func_filename = os.path.basename(func.__code__.co_filename)  # 取出方法所在的文件名
        # 如果有这个文件，并且已经配置了方法名对应的说明链接，则打印出来
        if func_filename in instruction_file_dict.keys() and instruction_file_dict[func_filename][func.__name__]:
            print(
                f'正在运行：office.{os.path.basename(func_filename)[:-3]}.{func.__name__} , 这个方法的使用说明：{instruction_file_dict[func_filename][func.__name__]}')
            print(SPLIT_LINE)
        instruction_res = func(*args, **kwargs)
        return instruction_res

    return instruction_wrapper


#############################################
# 以下是本文件的工具模块，用来更新方法和链接
from inspect import getmembers, isfunction, ismethod, isclass


# 获取模块包含的方法名
def get_method_name(file):
    for method_name in getmembers(file):
        if isfunction(method_name[1]):
            print(f'"{method_name[0]}":"",')


from office import api

if __name__ == '__main__':  # TODO:完善这个装饰器
    get_method_name(file=api.word)
